import subprocess
import sys

import click


@click.command()
@click.argument("name")
def cmd(name):
    """Check current branch's name."""
    result = subprocess.run(
        ["git", "rev-parse", "--abbrev-ref", "HEAD"],
        capture_output=True,
        encoding="utf-8",
    )
    if result.returncode:
        click.echo(result.stdout, nl=False)
        click.echo(result.stderr, nl=False, err=True)
        click.echo("FATAL: git did not exit cleanly.", err=True)
        sys.exit(3)

    got = result.stdout.rstrip()
    if name != got:
        click.echo(f"Branch name check failed: {name!r} != {got!r}", err=True)
        sys.exit(1)



"""Plug together the various commands with ``click``."""

import importlib

import click

commands = [
    # File system
    "directory-check-empty",
    # Git
    "git-check-branch",
    "git-check-clean",
    "git-check-remote",
    "git-check-tag",
    # Version handling
    "version-check-validity",
    "version-bump",
]


class _SubpackageMultiCommand(click.MultiCommand):
    def list_commands(self, ctx):
        # This is the ordered list of the commands.
        return commands

    def get_command(self, ctx, name):
        assert name in commands
        identifier = name.replace("-", "_")

        module = importlib.import_module(f"release_helper.commands.{identifier}")
        return module.cmd


def main():
    """Primary entry point from the command line."""
    cli = _SubpackageMultiCommand(name="release=helper", no_args_is_help=True)
    cli()



import subprocess
import sys

import click


@click.command()
@click.argument("name")
@click.argument("urls", nargs=-1)
def cmd(name, urls):
    """Check remote includes given URL(s)."""
    if not urls:
        click.echo("FATAL: Got no URLs.", err=True)
        sys.exit(2)

    result = subprocess.run(
        ["git", "remote", "get-url", "--push", name],
        capture_output=True,
        encoding="utf-8",
    )
    if result.returncode:
        click.echo(result.stdout, nl=False)
        click.echo(result.stderr, nl=False, err=True)
        click.echo("FATAL: git did not exit cleanly.", err=True)
        sys.exit(3)

    got_urls = result.stdout.rstrip().splitlines()

    if not (set(got_urls) & set(urls)):
        formatted_urls = "\n- " + "\n- ".join(urls)
        click.echo(
            f"git remote {name!r} does not include any URL from:{formatted_urls}",
            err=True,
        )
        sys.exit(1)



import subprocess
import sys

import click


def _is_clean():
    result = subprocess.run(
        ["git", "status", "--porcelain"], capture_output=True, encoding="utf-8",
    )
    return result.stdout.strip() == ""


@click.command()
def cmd():
    """Check that the repository is in a clean state."""
    if not _is_clean():
        click.echo("git working directory/index is not clean.", err=True)
        sys.exit(1)



import subprocess
import sys

import click


@click.command()
@click.argument("name")
@click.option("--exists/--does-not-exist", required=True)
def cmd(name, exists):
    """Check that given tag (does not) exist."""
    result = subprocess.run(
        ["git", "show-ref", "--tags", "--quiet", "--verify", "--", f"refs/tags/{name}"],
        capture_output=True,
        encoding="utf-8",
    )

    if exists and result.returncode:
        click.echo(f"Expected tag {name!r} to exist.", err=True)
        sys.exit(1)
    elif not exists and not result.returncode:
        click.echo(f"Expected tag {name!r} to not exist.", err=True)
        sys.exit(1)



import sys

import click
from packaging.utils import canonicalize_version
from packaging.version import InvalidVersion, Version


@click.command()
@click.argument("version", required=True)
@click.option("--require-canonical", is_flag=True)
def cmd(version, require_canonical):
    """Ensure a version is a valid PEP 440 version."""
    try:
        parsed_version = Version(version)
    except InvalidVersion:
        click.echo(f"{version!r} is not a valid PEP 440 version.", err=True)
        sys.exit(1)

    if require_canonical:
        canonical_version = canonicalize_version(parsed_version)
        if version != canonical_version:
            click.echo(
                f"{version!r} is not a canonical PEP 440 version "
                f"(should be {canonical_version!r}).",
                err=True,
            )
            sys.exit(1)



import re
import sys
from pathlib import Path

import click


@click.command()
@click.argument(
    "file",
    required=True,
    type=click.Path(
        exists=True, file_okay=True, dir_okay=False, writable=True, readable=True
    ),
)
@click.option(
    "--variable-name",
    default="__version__",
    type=str,
)
@click.argument("version_string", required=True)
def cmd(file, variable_name, version_string):
    """Bump version in given file.

    If this is a `.py` file, replace the version in an assignment of the given version.
    If this is a `.txt` file, write the version with a newline into the file.
    Errors out otherwise.
    """
    path = Path(file)
    if path.suffix == ".py":
        # Find and replace the __version__ = "..." line.
        contents = path.read_text()
        new_contents = re.sub(
            f'{variable_name} = "(.+)"',
            f'{variable_name} = "{version_string}"',
            contents,
        )
        if new_contents == contents:
            click.echo("Did not make any changes to the file.", err=True)
            sys.exit(1)
        path.write_text(new_contents)
    elif path.suffix == ".txt":
        path.write_text(version_string + "\n")
    else:
        click.echo(f"Don't know what to do for {path.suffix} files", err=True)
        sys.exit(1)



"""Ensure a directory is empty, if it exists."""

import os
import shutil
import sys

import click


def _delete_contents(path):
    for root, dirs, files in os.walk(path):
        for f in files:
            os.unlink(os.path.join(root, f))
        for d in dirs:
            shutil.rmtree(os.path.join(root, d))


def _handle_one(path, delete):
    if not os.path.exists(path):
        return

    if not os.path.isdir(path):
        return "is not a directory"

    contents = os.listdir(path)
    if not contents:  # nothing inside, all OK
        return

    if not delete:  # this has files, and not supposed to delete them.
        return "is not empty"

    _delete_contents(path)


@click.command()
@click.option(
    "--delete",
    is_flag=True,
    default=False,
    help="Delete the contents of given directories, if non-empty.",
)
@click.argument("directories", required=True, nargs=-1)
def cmd(directories, delete):
    """Ensure a directory is empty, if it exists."""
    issues = []
    for path in directories:
        reason = _handle_one(path, delete)
        if reason is not None:
            issues.append((path, reason))

    if issues:
        for path, reason in issues:
            click.echo(f"{path} {reason}.", err=True)
        sys.exit(1)
